@@ -19,7 +19,7 @@ use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG} || $ENV{SWAGGER2_DEBUG} || 0;
 use constant WARN_ON_MISSING_FORMAT => $ENV{JSON_VALIDATOR_WARN_ON_MISSING_FORMAT}
   || $ENV{SWAGGER2_WARN_ON_MISSING_FORMAT} ? 1 : 0;
 
-our $VERSION = '0.69';
+our $VERSION = '0.70';
 our @EXPORT_OK = qw(validate_json);
 
 my $HTTP_SCHEME_RE = qr{^https?:};
@@ -517,17 +517,21 @@ sub _validate_type_object {
     push @{$rules{$_}}, $r for grep { $_ =~ /$p/ } keys %$data;
   }
 
-  # special case used internally
-  $rules{id} ||= [{type => 'string'}] if !$path and $data->{id};
   $additional = exists $schema->{additionalProperties} ? $schema->{additionalProperties} : {};
-
   if ($additional) {
     $additional = {} unless ref $additional eq 'HASH';
     $rules{$_} ||= [$additional] for keys %$data;
   }
-  elsif (my @keys = grep { !$rules{$_} } keys %$data) {
-    local $" = ', ';
-    return E $path, "Properties not allowed: @keys.";
+  else {
+    # Special case used internally when validating schemas: This module adds "id"
+    # on the top level which might conflict with very strict schemas, so we have to
+    # remove it again unless there's a rule.
+    local $rules{id} = 1 if !$path and exists $data->{id};
+
+    if (my @keys = grep { !$rules{$_} } keys %$data) {
+      local $" = ', ';
+      return E $path, "Properties not allowed: @keys.";
+    }
   }
 
   for my $k (keys %required) {
@@ -725,7 +729,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-0.69
+0.70
 
 =head1 SYNOPSIS
 
